#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#include "net_connect.h"
#include "mqtt_pub.h"

/* Private data. */
static WiFiClient espClient;
static PubSubClient client(espClient);
static const char *client_name;
static const char *topic_base;

/* Private functions. */
static void setup_wifi(void);
static void reconnect(void);

void
mqtt_setup(const char *client_name_a, const char *topic_base_a)
{
	client_name = client_name_a;
	topic_base = topic_base_a;
	setup_wifi();
	client.setServer(mqtt_broker, mqtt_port);
}

void
mqtt_loop()
{
	if (!client.connected()) {
		reconnect();
	}
	client.loop();
}

void
mqtt_publish(const char *topic_name, const char *value)
{
	String t = topic_base;
	t.concat('/');
	t.concat(topic_name);
	client.publish(t.c_str(), value, true);
}

static void
setup_wifi(void)
{
	delay(10);
	// We start by connecting to a WiFi network
	Serial.println();
	Serial.print("Connecting to ");
	Serial.println(wifi_ssid);

	WiFi.begin(wifi_ssid, wifi_password);

	while (WiFi.status() != WL_CONNECTED) {
		delay(500);
		Serial.print(".");
	}

	Serial.println("");
	Serial.println("WiFi connected");
	Serial.println("IP address: ");
	Serial.println(WiFi.localIP());
}

static void
reconnect(void)
{
	// Loop until we're reconnected
	while (!client.connected()) {
		Serial.print("Attempting MQTT connection to ");
		Serial.print(mqtt_broker);
		Serial.print(':');
		Serial.print(mqtt_port);
		Serial.print(" as ");
		Serial.print(client_name);
		// Attempt to connect
		if (client.connect(client_name)) {
			Serial.println(" - connected");
		} else {
			Serial.print(" - failed, rc=");
			Serial.print(client.state());
			Serial.println(" try again in 5 seconds");
			delay(5000);
		}
	}
}
